#include <msp430.h> 


// P1.0 = L1-Yellow
// P1.1 = L1-Green
// P1.2 = L1-Red

// P1.3 = L2-Yellow
// P1.4 = L2-Green
// P1.5 = L2-Red

// P1.6 = board LED - signalization of change from operation to yellow-flash

// P2.6 = button (connected to GND)


void wait001(unsigned long int i_wt);
void m_loop();
void i_loop();
void post_i_loop();

unsigned int butt_act;

int main(void) {
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	P1DIR = 0xFF;					// set P1.0 - P1.7 to output
	P2DIR = 0x00;					// set P2.x to input
	P2REN = 0x40;					// enable pull-up resistor for P2.6

	P1SEL = 0x00;
	P2SEL = 0x00;


	P1OUT = 0x00;					//all LEDs off
	P2OUT = 0xFF;

	for(;;)
    	{
			P1OUT &= 0b10111111;	//transition LED off
			m_loop();				//execute main working loop - traffic light are working
			P1OUT &= 0b10111111;	//transition LED off
			i_loop();				//execute idle loop - yellow light is flashing
			post_i_loop();
    	}

}

void post_i_loop() {
	unsigned int i;
// post "off" phase - just wait a while to void switch to the i_loop by pressing the button too long

	i = 3;
	do {
		i--;
		P1OUT |= 0b00001001;		//L1 & L2-Yellow on
		wait001(30000);
		P1OUT &= 0b11110110;		//L1 & L2-Yellow off
		wait001(30000);
	   }
	while(i != 0);
}

void i_loop() {
	unsigned int i;
// first phase - keep the red on main road on first

	P1OUT &= 0b11011111;		//L2-Red off
	i = 5;
	do {
		i--;
		P1OUT |= 0b00001000;		//L2-Yellow on
		wait001(30000);
		P1OUT &= 0b11110111;		//L2-Yellow off
		wait001(30000);
	   }
	while(i != 0);

// second / main phase - keep just the yellow lights flashing

	P1OUT = 0x00;					//switch everything off
	butt_act = 0;					// default - button is not active
	for(;;) {
		P1OUT |= 0b00001001;		//L1-Yellow on & L2-Yellow on
		wait001(30000);
		P1OUT = 0x00;				//switch all lights off
		wait001(30000);
		if (butt_act == 1) return;
	}
}

void m_loop() {
	butt_act = 0;					// default - button is not active
	P1OUT = 0x00;					//switch all lights off
	for(;;) {
		P1OUT |= 0b00100100;		//L1-Red on & L2-Red on
		wait001(100000);
		if (butt_act == 1) return;
		P1OUT |= 0b00000001;		//L1-Yellow on
		wait001(100000);
		P1OUT &= 0b11111010;		//L1-Yellow off & L1-Red off
		P1OUT |= 0b00000010;		//L1-Green on
		wait001(400000);
		P1OUT &= 0b11111101;		//L1-Green off
		P1OUT |= 0b00000001;		//L1-Yellow on
		wait001(100000);
		P1OUT &= 0b11111110;		//L1-Yellow off
		P1OUT |= 0b00000100;		//L1-Red on
		wait001(100000);
		if (butt_act == 1) return;

		P1OUT |= 0b00001000;		//L2-Yellow on
		wait001(100000);
		P1OUT &= 0b11010111;		//L2-Yellow off & L2-Red off
		P1OUT |= 0b00010000;		//L2-Green on
		wait001(400000);
		P1OUT &= 0b11101111;		//L2-Green off
		P1OUT |= 0b00001000;		//L2-Yellow on
		wait001(100000);
		P1OUT &= 0b11110111;		//L2-Yellow off
	}
}
void wait001(unsigned long int i_wt) {
	volatile unsigned long int i;
	i = i_wt;
	do {
		i--;
		if ((P2IN & 0x40) != 0x40)
      	  {
    	  	  butt_act = 1;
    	  	  P1OUT |= 0b01000000;		//switch the board LED on - transition active
      	  }
	   }
	while(i != 0);

	return;
}
